function [cNeurons, distNeuronsToData, vLabels, iCycle,dRunTime] = bng_relational_ny_scl( ny_Dis, n, epochs, lambda0 )

%
% Relational Batch Neural Gas with Nystroem approximation
%
% Non-euclidean interpretation of dual cost function, distances are not squared before
%
% Params:
% Distances ... (m,m)-Matrix of pairwise dissimilarities (apply .^2 for Euclidean datasets)
% n ... Number of Neurons
%
% Returns:
% cNeurons ... Coefficient (n x m)-Matrix for Neuron Representation
% distNeuronsToData ... (n x m)-Matrix of distances between neurons and data points
% vLabels  ... assigned cluster numbers per entry 1 x m
% cycle the algorithm converged
%
% A. Hasenfuss (c) 2008, modified by A. Gisbrecht and Frank-Michael Schleif (2011-2012)

% Check parameters
error(nargchk(3, 4, nargin));

% Get sizes
m = size( ny_Dis{1}, 1 );

% Set parameters
if nargin<4 || isempty(lambda0) || isnan(lambda0), lambda0 = n/2; end

% lambda
lambda = lambda0 * (0.01/lambda0).^((0:(epochs-1))/epochs);

% Init and normalize
cNeurons   = rand(n,m);
cNeurons   = cNeurons ./ (sum(cNeurons,2) * ones(1,m));
U          = ny_Dis{1};
Ut         = U';
T          = ny_Dis{2} * Ut;
% Do it...
initLabels = ceil(rand(1,m)*n);
vLabels    = initLabels;
last       = 0;
iCycle     = 1;

dCPUTime   = cputime;

temp1 = (cNeurons * U)*T;
temp2 = 0.5*sum(bsxfun(@times,temp1,cNeurons),2);
distNeuronsToData = max(bsxfun(@minus,temp1,temp2),realmin);   
while(any(vLabels ~= last) && iCycle < epochs)
  % Determine dissimilarities between neurons and data
  Time = cputime;
  
  % Determine ranking (k_ij)
  [~, idx]    = sort( distNeuronsToData, 1 );
  [~,Ranking] = sort(idx);

  % Calculate h(rank)
  Hvector = exp(-(0:(n-1))/lambda(iCycle));  

  % Calculate h_lamdba(k_ij) matrix and normalize
  HH = Hvector(Ranking);

  % Update rule - providing new alpha values (~cNeurons)
  cNeurons = bsxfun(@rdivide,HH,max(sum(HH,2),realmin));

  % Determine distance between final neurons and data
  distNeuronsToData = max(distance_neurons_data_ny( U,T, cNeurons ),realmin);
  
  % label assignments
  last              = vLabels;
  vLabels           = argmin(distNeuronsToData);
  
  iCycle=iCycle+1;
  
  if(mod(iCycle,10)==0)
      fprintf('non corresp: %f  cycles: %f time: %f\n', sum(vLabels ~= last)/m, iCycle/epochs, cputime-Time);
  end
end
dRunTime = cputime-dCPUTime;
   