%
% Transforms a similarity (aka kernel matrix) into a nystroem approximated
% dissimilarity matrix without, calculating the full kernel matrix
%
% Input a  
%   fKernel        - pointer to a kernel e.g. fKernel = @(X,Y)X*Y'; % linear kernel
%   Data           - N x dim 
%   dPercentage    - (0,1] - sampling for nystroem
%
%   Transformation of Similarity to Dissimilarity in acc. to Pekalska Duin
%   p. 111, Theorem 3.18 - giving squared dissimilarities (second case)
%
% Output 
%    ny_Dis      - cellarray {1} - k x N - dissimilarities, {2}, k x k - pinv(dis(vP,vP))
%                  of landmark points vP = k x 1
%    vP          - used landmarks
%
function [ny_Dis vP] =sim2dis_ny(fKernel,Data,dPercentage)
[N,dim] = size(Data);
P       = ceil(N*dPercentage);
if(P > 5000)
    error('To many landmarks - pinv will be to slow - use low dPercentage');
end
vP      = randsample(N,P);   % nystroem landmarks

Sim     = fKernel(Data(vP,:),Data);            % subsampled similarities - k x N
vDiags  = zeros(N,1); 

% check if diag is constant
for(k=1:N)
    vDiags(k)  = fKernel(Data(k,:),Data(k,:)); % get diagonals 
end

% normalization - regular - optional
if(N*P > 1e10)
    Dis=zeros(P,N);    
    for i=1:P
        for j=1:N
            Dis(i,j)=vDiags(vP(i)) + vDiags(j) -2*Sim(i,j);
        end
    end    
else
    Dis=bsxfun(@plus,bsxfun(@plus,-2*Sim,vDiags(vP)),vDiags');
end

ny_Dis{1} = Dis';
ny_Dis{2} = pinv(Dis(:,vP));


