function [Partition] = crossvalidation( k, Labels )

%
% k-fold Cross Validation
%
% Divides the data set of size n into k blocks with fair label distribution
%
% Returns an (n x 1)-array containing the class index from {1,...,k} of
% every data point within the partition.
%
% Usage: Create logical indexes for training and test set
%    [Partition] = crossvalidation( k, DataSize, Labels );
%    for i = 1:k
%      test_logical_idx = (Partition == i);
%      train_logical_idx = (Partition ~= i);
%
%      TrainData = Data( train_logical_idx, : );
%      TestData = Data( test_logical_idx, : );
%
%       % FOR PROXIMITY DATA:
%       TrainDist = Distances( train_logical_idx, train_logical_idx );
%       TestDist = Distances( test_logical_idx, test_logical_idx );
%       BetweenDist = Distances( train_logical_idx, test_logical_idx );
%      
%      TrainLabels = Labels( train_logical_idx, : );
%      TestLabels = Labels( test_logical_idx, : );
%       
%      // do something... 
%
%    end
%
% Alternative: You may also use the following functions...
%    [Partition] = crossvalidation( k, DataSize, Labels );
%    for i = 1:k
%      [TrainData, TestData] = crossvalidation_divide_data( Data, Partition, j);
%        % FOR PROXIMITY DATA:
%        [TrainDist, TestDist, BetweenDist] = crossvalidation_divide_distances(Distances, Partition, j );
%      [TrainLabels, TestLabels] = crossvalidation_divide_labels( Labels, Partition, j );
%
%      // do something... 
%
%    end
%
% A. Hasenfuss (c) 2006-2007
%

% Check parameter
n = numel(Labels);

% To store results
Partition = zeros( n, 1 );

% Analyse data
T = tabulate( Labels );
T( T(:,2) == 0,: ) = [];
num_classes = size(T,1);

% For every class...
for i = 1:num_classes
  
  % Get indexes
  idx = find( Labels == T(i,1) );
  length = numel(idx);
  
  % Create array 
  num = floor( length / k );
  
  if num ~= 0
    L = zeros(length,1);
  
    for j = 1:k
      pos = (j-1)*num + 1;
      L( pos:pos+num-1 ) = j;        
    end

  end
  
  % Care for the rest (at most k-1 elements)
  diff = length - num*k;
  if diff ~= 0
    R = randperm( k );
    L( length-diff+1 : length ) = R(1:diff);
  end
  
  % Randomize
  L = L( randperm(length) );
  
  % Store
  Partition( idx ) = L;
  
end




