function pipeline( DatasetName, Data, Labels, iNrOfPrototypes, epochs, kfold, runs, dNystroem, fKernel )

%
% Pipeline to evaluate Fast-Approximate-SCL
% DatasetName - string
% Data        - N x N matrix (relational) or N x d (matrix data) 
% Labels      - N x 1 - numeric labels
% iNrOfPrototypes - scalar > 0
% epochs      - upper runtime limit (e.g. 100) 
% kfold       - number of CV folds
% runs        - number of repetitions 
% dNystroem   - nystroem approximation e.g. 0.1
% fKernel     - if given its assumed that D is a N x d matrix of measurements (not relational) - a valid kernel pointer
%

sMethod = 'fast_approx_scl';

% Global
global DEBUG;
DEBUG = false;

% Needed values
m = size( Data, 1 );

% Info output
disp(sprintf(' ** Using %d-fold crossvalidation on a %d-element dataset', kfold, m ));

bRelational = true;
if(exist('fKernel','var'))
    bRelational = false;
else
    Data = fCheckDissimilarity(Data); % check and try to fix if necessary
end

% Local Parameters
ClassificationAccuracyTest = zeros(runs, kfold);
ClassificationAccuracyTrain = zeros(runs, kfold);
NormMutualInf = zeros(runs,kfold);
Cycles = zeros(runs,kfold);

MeanClassificationAccuracyTrain = zeros(runs,1);
MeanClassificatioAccuracyTest = zeros(runs,1);
StdClassificationAccuracyTrain = zeros(runs,1);
StdClassificationAccuracyTest = zeros(runs,1);

MeanDualTrain = zeros(runs,1);
MeanDualTest = zeros(runs,1);
StdDualTrain = zeros(runs,1);
StdDualTest = zeros(runs,1);

% To store results
DualCostTrain = zeros(runs, kfold);
DualCostTest = zeros(runs, kfold);
compTime = zeros( runs, kfold );
numNeurons = zeros(runs, kfold);
vSparsity = zeros(runs, kfold); 
%% Do the runs
 for i = 1:runs

  % Cross-Validation
  [Partition] = crossvalidation( kfold, Labels );
  
  for j = 1:kfold
    test_logical_idx = (Partition == j);
    train_logical_idx = (Partition ~= j);
    TrainLabels = Labels( train_logical_idx, : );
    TestLabels = Labels( test_logical_idx, : );
  
    if(bRelational)
        TrainDist   = Data( train_logical_idx, train_logical_idx );
        TrainData   = TrainDist;
        TestDist    = Data( test_logical_idx, test_logical_idx );   % note ! diagonals are const for all X
        fKDistPtr   = @(X,Y)max(TestDist(X,Y),realmin); 
        BetweenDist = Data( train_logical_idx, test_logical_idx );
        % fast_approx_scl makes the nystroem approx on the training data (mainly for speed in the training)
        [Neurons, distNeuronsToTrainData, vAssignmentsTrain, iCycle, iCompTime, dDual] = fast_approx_scl(TrainData,iNrOfPrototypes,dNystroem,epochs);        % with ny
        distNeuronsToTestData = distance_neurons_to_testdata( BetweenDist, TrainDist, Neurons );                                                             % without ny
    else        
        TrainData       = Data(train_logical_idx,:);
        fKernelNyApprox = NystroemKernelPtr(Data,fKernel,dNystroem);
        DataIndexObject = find(test_logical_idx)';
        vDiags          = arrayfun(@(i)fKernelNyApprox(i,i),DataIndexObject);
        fKDistPtr       = @(X,Y)max(bsxfun(@minus,vDiags(X)+ vDiags(Y),2*fKernelNyApprox(DataIndexObject(X),DataIndexObject(Y))),realmin); 
        % fast_approx_scl makes the nystroem approx on the training data
        [Neurons, distNeuronsToTrainData, vAssignmentsTrain, iCycle, iCompTime, dDual] = fast_approx_scl(TrainData,iNrOfPrototypes,dNystroem,epochs,fKernel); % with ny
        distNeuronsToTestData = distance_neurons_to_testdata_ny( Data, train_logical_idx, test_logical_idx, fKernel, Neurons, dNystroem);                     % with ny
    end

    vAssignmentsTest    = assign_datapoints_to_neurons( distNeuronsToTestData );
   
    % Dual Cost Function
    vSparsity( i, j)      = nnz(Neurons)./numel(Neurons);
    DualCostTrain( i, j)  = dDual;                                                        % with ny
    DualCostTest(  i, j)  = determine_cost_function_dual( fKDistPtr, vAssignmentsTest );  % for relational without ny

    % post label neurons
    NeuronLabels          = label_neurons( distNeuronsToTrainData, TrainLabels );
    vValid                = ~isnan(NeuronLabels);
    
    % determine predicted labels for test data
    cLabels            = label_data( distNeuronsToTestData(vValid,:), NeuronLabels(vValid) );
    
    % Measure normalized mututal information
    NormMutualInf(i,j) = nmi(TestLabels,cLabels);
    Cycles(i,j)        = iCycle;
    compTime(i,j)      = iCompTime;
    
    ClassificationAccuracyTrain(i,j) = CalculatePostLabelAccuracy(vAssignmentsTrain, TrainLabels);
    dAcc          = sum(NeuronLabels(vAssignmentsTest)==TestLabels)./numel(TestLabels);
    ClassificationAccuracyTest(i,j)  = dAcc;%CalculatePostLabelAccuracy(vAssignmentsTest, TestLabels);
    
    disp(sprintf('* Done (n = %d)...dualqErrTrain %f, dualqErrTest %f, err-Train %f, err-Test %f',...
          iNrOfPrototypes, DualCostTrain(i,j), DualCostTest(i,j), ClassificationAccuracyTrain(i,j),ClassificationAccuracyTest(i,j)));     

  %% End of for j 
  end 

  %% Calculate and store results
   MeanClassificationAccuracyTrain(i,:) = mean(ClassificationAccuracyTrain(i,:));
   MeanClassificatioAccuracyTest(i,:) = mean(ClassificationAccuracyTest(i,:));
   StdClassificationAccuracyTrain(i,:) = std(ClassificationAccuracyTrain(i,:));
   StdClassificationAccuracyTest(i,:) = std(ClassificationAccuracyTest(i,:));

   MeanDualTrain(i,:) = mean(DualCostTrain(i,:));
   MeanDualTest(i,:) = mean( DualCostTest(i,:));
   StdDualTrain(i,:) = std( DualCostTrain(i,:));
   StdDualTest(i,:) = std(DualCostTest(i,:));

 %% End of for i
 end
  
  mHitrate = mean( MeanClassificatioAccuracyTest, 1 );
  mStdHitrate = std( MeanClassificatioAccuracyTest, 1 );
  
  mDualTrain = mean( MeanDualTrain, 1 );
  mStdDualTrain = std( StdDualTrain, 1 );

  mDualTest = mean( MeanDualTest, 1 );
  mStdDualTest = std( StdDualTest, 1 );
  
  disp(sprintf('Done %s#%d',DatasetName,m));
  disp(sprintf(' * Done dcTrain%.3f (%.3f) dcTest%.3f (%.3f) accTest%.3f (%.3f)',...
      mDualTrain, mStdDualTrain, mDualTest, mStdDualTest,...
      mHitrate, mStdHitrate ));
   
  
  % Write data to workspace
  assignin( 'base', 'Method', sMethod );
  assignin( 'base', 'AccuracyTrain', ClassificationAccuracyTrain );
  assignin( 'base', 'AccuracyTest', ClassificationAccuracyTest );
  assignin( 'base', 'NormMutualInf', NormMutualInf );
  assignin( 'base', 'Cycles', Cycles );

  assignin( 'base', 'MeanAccuracyTrain', MeanClassificationAccuracyTrain );
  assignin( 'base', 'MeanAccuracyTest', MeanClassificatioAccuracyTest );
  assignin( 'base', 'StdAccuracyTrain', StdClassificationAccuracyTrain );
  assignin( 'base', 'StdAccuracyTest', StdClassificationAccuracyTest );
  assignin( 'base', 'LastModel', Neurons);
  assignin( 'base', 'vSparsity',vSparsity);
  assignin( 'base', 'numNeurons', numNeurons );
  assignin( 'base', 'compTime', compTime );

% Save results
  filename = sprintf('ComparisonFastSCL_%s_epo%d_kfold%d_runs%d__%s.mat',...
    DatasetName,epochs,kfold,runs,datestr(clock,30));
  clear('Distances','Labels');
  if( ~exist('./results','dir') )
    mkdir('./results');
  end
  save(sprintf('%s%s',strcat('results',filesep),filename),'-v7.3');

