%
% Transforms a similarity (aka kernel matrix) into a nystroem approximated
% dissimilarity matrix without, calculating the full kernel matrix
%
% Input a  
%   fKernel        - pointer to a kernel e.g. fKernel = @(X,Y)X*Y'; % linear kernel
%   Data           - N x dim 
%   vRowIndices    - Nr x 1 
%   vColIndices    - Nc x 1 
%
%   Transformation of Similarity to Dissimilarity in acc. to Pekalska Duin
%   p. 111, Theorem 3.18 - giving squared dissimilarities (second case)
%
% Output 
%   Dis - dissimilarity matrix Nr x Nc
%
function [Dis] =sim2dis_indexed(fKernel,Data, vRowIndices, vColIndices)
[N,dim] = size(Data);

Sim     = fKernel(Data(vRowIndices,:),Data(vColIndices,:));            % subsampled similarities - k x N
vDiags  = zeros(N,1); 
iNrRows = numel(vRowIndices);
iNrCols = numel(vColIndices);
% check if diag is constant
for(k=1:N)
    vDiags(k)  = fKernel(Data(k,:),Data(k,:)); % get diagonals 
end

% normalization - regular - optional
if(iNrRows * iNrCols > 1e10)
    Dis=zeros(iNrRows,iNrCols);    
    for i=1:iNrRows
        for j=1:iNrCols
            Dis(i,j)=vDiags(vRowIndices(i)) + vDiags(vColIndices(j)) -2*Sim(i,j);
        end
    end    
else
    Dis=bsxfun(@plus,bsxfun(@plus,-2*Sim,vDiags(vRowIndices)),vDiags(vColIndices)');
end
